/*
 *    Copyright 2008-2010 Simone Tripodi
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.googlecode.injectlet.core.servlet;

import java.util.List;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.googlecode.injectlet.core.annotations.InjectorId;

/**
 * Abstract {@link ServletContextListener} that builds the {@link Injector}
 * and set it in the {@link ServletContext}.
 *
 * {@link ServletContext} init parameter will be bound to the
 * {@link Injector} through {@link com.google.inject.name.Names}, using the
 * taxonomy <code>servletContext.initParameter.&lt;paramName&gt;</code>.
 * The scope of these init parameter will be obviously global for all servlets.
 *
 * @author Simone Tripodi
 * @version $Id: AbstractGuiceServletContextListener.java 442 2010-06-02 20:09:12Z simone.tripodi $
 */
public abstract class AbstractGuiceServletContextListener implements ServletContextListener {

    /**
     * The log interface
     */
    private final Log log = LogFactory.getLog(this.getClass());

    private final String injectorId;

    /**
     * Return this class logger.
     *
     * @return this class logger.
     */
    protected Log getLog() {
        return this.log;
    }

    /**
     * 
     * @param modules
     * @deprecated
     */
    @Deprecated
    public AbstractGuiceServletContextListener(final Module... modules) {
        throw new IllegalStateException("Invoked constructor has been deprecated");
    }

    /**
     * Builds internally a new {@link Injector} that will be stored in the
     * {@link ServletContext} once <code>contextInitialized</code> method
     * will be called
     *
     * @param modules The modules will be used in the webapp
     */
    public AbstractGuiceServletContextListener() {
        if (this.getClass().isAnnotationPresent(InjectorId.class)) {
            this.injectorId = this.getClass().getAnnotation(InjectorId.class).value();
        } else {
            this.injectorId = Injector.class.getName();
        }
    }

    /**
     * 
     * @return
     */
    protected abstract List<Module> createContextModules();

    /**
     * Remove the {@link Injector} from the {@link ServletContext}
     *
     * @param servletContextEvent The event for notifications about changes to
     *      the servlet context of a web application.
     */
    public final void contextDestroyed(final ServletContextEvent servletContextEvent) {
        ServletContext servletContext = servletContextEvent.getServletContext();
        servletContext.removeAttribute(this.injectorId);
    }

    /**
     * Build the {@link Injector} and set it in the {@link ServletContext}
     *
     * @param servletContextEvent The event for notifications about changes to
     *      the servlet context of a web application.
     */
    public final void contextInitialized(final ServletContextEvent servletContextEvent) {
        final ServletContext servletContext = servletContextEvent.getServletContext();

        List<Module> modules = this.createContextModules();
        Injector injector = Guice.createInjector(modules);
        servletContext.setAttribute(this.injectorId, injector);
    }

}
