/*
 *    Copyright 2008-2010 Simone Tripodi
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.googlecode.injectlet.core.servlet;

import javax.servlet.Filter;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.inject.Injector;
import com.googlecode.injectlet.core.annotations.FilterInitParam;
import com.googlecode.injectlet.core.annotations.ServletContextInitParam;
import com.googlecode.injectlet.core.utils.InjectorUtils;

/**
 * The abstract injectable {@link Filter} based on {@link Injector}.
 *
 * Init parameters declared in {@link FilterConfig} are bound to the
 * {@link Injector} through {@link com.google.inject.name.Names}, using the
 * taxonomy <code>filterConfig.initParameter.&lt;paramName&gt;</code>.
 * So it is possible use the {@link Injector} injection feature also using
 * the init parameters, avoiding the override of the <code>init</code> method.
 * The scope of these init parameter will be obviously local for this filter.
 *
 * @author Simone Tripodi
 * @version $Id: AbstractInjectableFilter.java 447 2010-06-02 20:28:18Z simone.tripodi $
 */
public abstract class AbstractInjectableFilter implements Filter {

    /**
     * The class logger.
     */
    private final Log log = LogFactory.getLog(this.getClass());

    /**
     * Return this class logger.
     *
     * @return this class logger.
     */
    protected Log getLog() {
        return this.log;
    }

    /**
     * Initialize the filter binding the init parameters declared in
     * {@link FilterConfig} to the {@link Injector} through
     * {@link com.google.inject.name.Names}, using the
     * taxonomy <code>filterConfig.initParameter.&lt;paramName&gt;</code>.
     * 
     * @param filterConfig the FilterConfig which from will be extracted the
     *      init parameters
     * @throws ServletException if the {@link Injector} won't found in the
     *      {@link ServletContext}
     */
    public final void init(final FilterConfig filterConfig) throws ServletException {
        Injector parentInjector = InjectorUtils.retrieveInjector(filterConfig);

        final AnnotationsClassVisitor visitor = new AnnotationsClassVisitor();
        visitor.registerHandler(FilterInitParam.class, new FilterInitParamHandler(filterConfig));
        visitor.registerHandler(ServletContextInitParam.class, new ServletContextInitParamHandler(filterConfig.getServletContext()));

        Injector injector = parentInjector.createChildInjector(new InitParamModule(visitor, this.getClass()));

        if (this.log.isDebugEnabled()) {
            this.log.debug("Injecting memebers on Filter ["
                    + filterConfig.getFilterName()
                    + "]");
        }

        injector.injectMembers(this);

        if (this.log.isDebugEnabled()) {
            this.log.debug("Filter ["
                    + filterConfig.getFilterName()
                    + "] successfully intialized");
        }
    }

}
