/*
 *    Copyright 2008-2010 Simone Tripodi
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.googlecode.injectlet.core.utils;

import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import com.google.inject.Binder;
import com.google.inject.Injector;
import com.googlecode.injectlet.core.servlet.AbstractGuiceServletContextListener;

/**
 * This class contains some reusable {@link Injector} utils methods.
 *
 * @author Simone Tripodi
 * @version $Id: InjectorUtils.java 442 2010-06-02 20:09:12Z simone.tripodi $
 * @since 2.0
 */
public final class InjectorUtils {

    /**
     * The {@link Injector} id set from an
     * {@link AbstractGuiceServletContextListener} instance.
     */
    private static final String INJECTOR_ID = "injector-id";

    /**
     * This class can't be instantiated.
     */
    private InjectorUtils() {
    }

    /**
     * Build a child {@link Injector} retrieving the parent from the Servlet
     * Context and binding the servlet init-params as {@link Binder} constants.
     *
     * @param servlet the servlet for whom the {@link Injector} has to be created.
     * @return the new {@link Injector}.
     * @throws ServletException if it's not possible to create the {@link Injector}.
     */
    public static Injector retrieveInjector(final HttpServlet servlet) throws ServletException {
        return retrieveInjector(servlet.getServletConfig().getInitParameter(INJECTOR_ID), servlet.getServletContext());
    }

    /**
     * Build a child {@link Injector} retrieving the parent from the Servlet
     * Context and binding the filter init-params as {@link Binder} constants.
     *
     * @param filterConfig the filter config for whom the {@link Injector} has to be created.
     * @return the new {@link Injector}.
     * @throws ServletException if it's not possible to create the {@link Injector}.
     */
    public static Injector retrieveInjector(final FilterConfig filterConfig) throws ServletException {
        return retrieveInjector(filterConfig.getInitParameter(INJECTOR_ID), filterConfig.getServletContext());
    }

    /**
     * Retrieve the {@link Injector} from the Servlet Context identified by the
     * injector-id specified in a servlet/filter init param.
     *
     * If servlet/filter injector-id init param won't specified, the default
     * injector-id will be used.
     *
     * @param fromConfigInjectorId the init-param injetor-id.
     * @param context the servlet context from which extract the parent {@link Injector}
     * @return the {@link Injector} identified by the id.
     * @throws ServletException if the {@link Injector} is not found.
     */
    private static Injector retrieveInjector(final String fromConfigInjectorId,
            final ServletContext context) throws ServletException {
        String injectorId;
        if (fromConfigInjectorId == null) {
            injectorId = Injector.class.getName();
        } else {
            injectorId = fromConfigInjectorId;
        }

        Object injectorObject = context.getAttribute(injectorId);
        if (injectorObject == null) {
            throw new ServletException("Guice Injector with id ["
                    + injectorId
                    + "] not found in the Servlet Context");
        }

        return (Injector) injectorObject;
    }

}
